const dollarStub = /\$[0-9a-z]+/g
const fns = {}

const titleCase = function (str) {
  return str.replace(/\w\S*/g, (txt) => txt.charAt(0).toUpperCase() + txt.substring(1).toLowerCase())
}

// doc.replace('foo', (m)=>{})
const replaceByFn = function (main, fn) {
  main.forEach(m => {
    let out = fn(m)
    m.replaceWith(out)
  })
  return main
}

// support 'foo $0' replacements
const subDollarSign = function (input, main) {
  if (typeof input !== 'string') {
    return input
  }
  let groups = main.groups()
  input = input.replace(dollarStub, (a) => {
    let num = a.replace(/\$/, '')
    if (groups.hasOwnProperty(num)) {
      return groups[num].text()
    }
    return a
  })
  return input
}

fns.replaceWith = function (input, keep = {}) {
  let ptrs = this.fullPointer
  let main = this
  this.uncache()
  if (typeof input === 'function') {
    return replaceByFn(main, input)
  }
  let terms = main.docs[0]
  let isPossessive = keep.possessives && terms[terms.length - 1].tags.has('Possessive')
  // support 'foo $0' replacements
  input = subDollarSign(input, main)

  let original = this.update(ptrs)
  // soften-up pointer
  ptrs = ptrs.map(ptr => ptr.slice(0, 3))
  // original.freeze()
  let oldTags = (original.docs[0] || []).map(term => Array.from(term.tags))
  // slide this in
  if (typeof input === 'string') {
    input = this.fromText(input).compute('id')
  }
  main.insertAfter(input)
  // are we replacing part of a contraction?
  if (original.has('@hasContraction') && main.contractions) {
    let more = main.grow('@hasContraction+')
    more.contractions().expand()
  }
  // delete the original terms
  main.delete(original) //science.

  // keep "John's"
  if (isPossessive) {
    let tmp = main.docs[0]
    let term = tmp[tmp.length - 1]
    if (!term.tags.has('Possessive')) {
      term.text += '\'s'
      term.normal += '\'s'
      term.tags.add('Possessive')
    }
  }
  // what should we return?
  let m = main.toView(ptrs).compute(['index', 'lexicon'])
  if (m.world.compute.preTagger) {
    m.compute('preTagger')
  }
  // replace any old tags
  if (keep.tags) {
    m.terms().forEach((term, i) => {
      term.tagSafe(oldTags[i])
    })
  }
  // try to co-erce case, too
  if (keep.case && m.docs[0] && m.docs[0][0] && m.docs[0][0].index[1] === 0) {
    m.docs[0][0].text = titleCase(m.docs[0][0].text)
  }

  // try to keep some pre-post punctuation
  // if (m.terms().length === 1 && main.terms().length === 1) {
  //   console.log(original.docs)
  // }

  // console.log(input.docs[0])
  // let regs = input.docs[0].map(t => {
  //   return { id: t.id, optional: true }
  // })
  // m.after('(a|hoy)').debug()
  // m.growRight('(a|hoy)').debug()
  // console.log(m)
  return m
}

fns.replace = function (match, input, keep) {
  if (match && !input) {
    return this.replaceWith(match, keep)
  }
  let m = this.match(match)
  if (!m.found) {
    return this
  }
  this.soften()
  return m.replaceWith(input, keep)
}
export default fns
